function [start, stop] = findMainContour(ssmimg, varargin)
% [start, stop] = findMainContour(ssmimg, av, thresh)
% 
% calculate contour along main diagonal of self similarity matrix. It is
% forward oriented, i.e., stop and start will be monotonically increasing.
%
% INPUT
%   ssmimg ... self similarity matrix
%   av ....... numbers of frames to average over (default is 0 => no averaging)
%   thresh ... threshold on self similarity matrix (default is 0.3)
%
% OUTPUT
%   start ... vector containing start index for each frame
%   stop .... vector containing stop index for each frame
%
%  EXAMPLES
%    % no averaging, threshold at 0.2
%    [start, stop] = findMainContour(ssmimg, 0, 0.2);
%    % average over 10 frames, default threshold
%    [start, stop] = findMainContour(ssmimg, 10);
%


%set parameters
numvarargs = length(varargin);
optargs = {0, 0.3};
optargs(1:numvarargs) = varargin;
% [optargs{1:numvarargs}] = varargin{:}; % other way of putting it
[av, thresh] = optargs{:};


if av>0
  % average over av frames
  ssmimg = conv2(1/av^2*ones(av,av), ssmimg);
  ssmimg = ssmimg(ceil(av/2):end-floor(av/2),ceil(av/2):end-floor(av/2));
end


%initialize (output) parameters
nframes=size(ssmimg,1);
stop = zeros(size(ssmimg,1),1);
start = stop;

for k=1:nframes
  % find end points
  temp = ssmimg(k,k:end);
  qe = find(temp>thresh,1)+k-1;
  if isempty(qe)
    qe = nframes;
  else qe = qe-1;
  end%fi handle empty stop

  % find start points
  temp = (ssmimg(k,1:k));
  temp = fliplr(temp);
  qs = find(temp>thresh,1);
  if isempty(qs)
    qs = 1;
  else qs = k-qs+2;
  end %fi handle empty start

  % make start and end points monotonically increasing
  for kkk = k-1:-1:1
    if qe < stop(kkk)
      stop(kkk) = qe;
    end
    if kkk==k-1 && qs < start(kkk)
      qs = start(kkk);
    end
  end % for make start and end monotonically increasing

  % set start and end points
  stop(k) = qe;
  start(k) = qs;

end %for go through every frame


% draw result if no output argument was given (DEBUGGING)
if nargout==0
  figure(2);
  imagesc(ssmimg,[0,2]); axis image;
  hold on;
  plot(stop, [1:nframes], 'Marker', '.', 'Color', 'r');
  plot(start, [1:nframes], 'Marker', '.', 'Color', 'g');
  hold off;
end

end %of function