function [rect_bwd, rect_fwd] = segmentationMethodA ( rectbwd, rectfwd, ignoreband, minblobdist )
%[rect_bwd, rect_fwd] = segmentationMethodA ( rectbwd, rectfwd, ignoreband, minblobdist )
%
% This method tries to find squares along a diagonal in a self similarity
% matrix (rectbwd and rectfwd come from such a matrix).
%
% Rectfwd and rectbwd come from the floodFillWithSquares method and carry
% the maximum backward and forward rectangle that fit into the data.
% THIS HAS BEEN CHANGED:
% Rectfwd and rectbwd come from the findMainContour method and carry
% row-wise start and end indices of the covered region in the data such 
% that each of the sequences are monotonically increasing in x and y.
%
% For rectbwd (looking backward only) the method works as follows:
% If there is a massive increase (discribed by ignoreband) in the left side
% (starting index) of two neighboring square AND this has not happened for
% over a certain number of frames (minblobdist) THEN save the current start
% and end index as a found square.
%
% For rectfwd (looking ahead only) the methods works the other way around.
% Instead of the left side of a square, we look at the right side (end
% index). Also we go through it from bottom to top.
%
%
% INPUT
%   rectbwd ....... sequence of squares, where a square is given by the
%                   the intervall it covers (start and end index). The
%                   first column contains the start indices and the second
%                   column the end indices.
%   rectfwd ....... sequence of squares
%   ignoreband .... defines how much difference there should be in two
%                   sequential squares in order to be counted as a relevant
%                   square (i.e. there is a 'major' change in the start/end index)
%   minblobdist ... minimum numbers of frames that should lie inbetween two
%                   relevant squares
%
% OUTPUT
%   rect_bwd ... found relevant squares in rectbwd
%   rect_fwd ... found relevant squares in rectfwd
%
% EXAMPLE USAGE:
%   % get forward and backward squares
%   [rectfwd, rectbwd] = floodFillWithSquares(ssmimg_binary);
%   % do THIS segmentation ignoring sequential squares that are less than 4
%   % frames apart from one another and also ignoring relevant squares when
%   % the do not lie over 10 frames apart.
%   [rect_temp_b, rect_temp_f] = segmentationMethodA(rectbwd, rectfwd, 4, 10);
%   % merge rectangeles (if they overlap each other by over 50 %)
%   rect_temp_b = mergeIntervals(0.5, {rect_temp_b});
%   rect_temp_f = mergeIntervals(0.5, {rect_temp_f});
%   % merge found forward and backward squares
%   rect = mergeIntervals(0.5, {rect_temp_b rect_temp_f});
% 

% juuust in case set some default values
if nargin == 2
  ignoreband = 6;
  minblobdist = 10;
elseif nargin == 3
  minblobdist = 10;
end


%% START handle bwd
old = 0;
idx = 1;
rectbwd(:,3) = [0; rectbwd(2:end,1)-rectbwd(1:end-1,1)];% difference in start indices

% look at each backward rectangle
for kk=2:size(rectbwd,1)

  shift_in_start_index = rectbwd(kk,1)-rectbwd(kk-1,1);
  % set type of change for the current pair of squares
  rectbwd(kk,4) = setTypeOfChange(shift_in_start_index, ignoreband);
  rectbwd(kk,5) = kk-1-old; %width-1

  % if the decrease in the start index is larger or equal than the 
  % set tolerance=ignoreband (or if it's the last frame)
  if (shift_in_start_index > ignoreband) || (kk==size(rectbwd,1))
    % save start index and how many frames since last massive decrease
    rect(1:2,idx) = [rectbwd(kk-1,1);kk-1-old]; 

    % if a certain number of frames has not had a massive decrease
    if kk-1-old > minblobdist

      % save current index pair (depending on wether it's the last frame or not)
      if (kk==size(rectbwd,1))
        rect (3:4,idx) = [rectbwd(kk-1,1); rectbwd(kk,2)]; % start and end index
      else
        rect (3:4,idx) = [rectbwd(kk-1,1); rectbwd(kk-1,2)]; % start and end index
      end %fi save current index pair

    end %fi last rectangle found 15 (or so) frames ago

    old = kk-1;
    idx = idx+1;

  end %fi start index changed by more than 6 (or whatever value there is after the '>=')

end% for look at each backward square

% if no second index pair (in rows 3 and 4) have been saved, 
if size(rect,1) < 4 
  % discard everything here
  rect_bwd = [];
% otherwise
else
  % use only found valid squares i.e. only those which far enough 
  % apart from one another
  rect_bwd = rect(3:4,:);
  rect_bwd = rect_bwd(find(rect_bwd));
  rect_bwd = reshape(rect_bwd,2,numel(rect_bwd)/2)';
end



%% START handle fwd
% reset values
rect = [];
old = 0;
idx = 1;
rectfwd(:,3) = [0; rectfwd(2:end,2)-rectfwd(1:end-1,2)]; % difference in start indices

% go through all forward squares
for kk=size(rectfwd,1)-1:-1:1
  
  shift_in_end_index = rectfwd(kk+1,2)-rectfwd(kk,2);
  old = old+1;

  % if the decrease in the end index is larger or equal than the set tolerance (or if it's the last frame)
  if shift_in_end_index > ignoreband || (kk==1) 
    % save start index and how many frames since last massive decrease
    rect(1:2,idx) = [rectfwd(kk+1,2);old]; %start index and width
    
    % if a certain number of frames has not had a massive decrease
    if old > minblobdist
      
      % save current index pair (depending on wether it's the first frame or not)
      if (kk==1)
        rect (3:4,idx) = [rectfwd(kk,1); rectfwd(kk+1,2)]; 
      else
        rect (3:4,idx) = [rectfwd(kk+1,1); rectfwd(kk+1,2)];
      end % fi save current index pair
      
    end % fi a certain number of frames nhas not had a massive decrease
    
    old = 0;
    idx = idx+1;
    
  end %fi start index changed by more than 6 (or whatever value there is after the '>=')
  
end %for go through all forward squares

% use only found valid squares i.e. only those which far enough 
% apart from one another
rect_fwd = fliplr(rect(3:4,:));
rect_fwd = rect_fwd(find(rect_fwd));
rect_fwd = reshape(rect_fwd,2,numel(rect_fwd)/2)';




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function v = setTypeOfChange(value_to_judge, ignoreband)
    if (value_to_judge == 0)
      v=1; %increase 
    elseif (value_to_judge == 1)
      v=0; %constant
    elseif (value_to_judge < ignoreband)
      v=-1; %decrease
    else
      v=-2; %massive decrease
    end
  end % of function setTypeOfChange

end % of function
