function [overlap_quality, overlap_width, offset_start_idx, offset_end_idx] = evaluateSegmentation (manual_cuts, computed_cuts, varargin )
% Evaluates the computed segmentation, i.e. compares the manual annotation
% with the computed segmentation.
%
% INPUT
%   manual_cuts ..... The hand annotated cuts (that were converted into
%                     squares defined by their start and end index).
%   computed_cuts ... The squares coming from the automated segmentation
%   varargin ........ see similarity_measure, thresh
%    similarity_measure ... defines the measure to use in order to evaluate
%                           the quality of the found segmentation. The two
%                           options are 
%                           'OverlapRatio' (needs the parameter 'thresh')
%                           'OverlapUnionRatio'
%                           'NumberOfSegments' 
%    thresh ............... (iff similarity measure is 'OverlapRatio'),
%                           threshold when this actually is evaluated. If
%                           squares do sufficiently (=> thresh) overlap,
%                           the overlap ratio will be set to 0.
%
% OUTPUT
%   overlap_quality .... matrix containing the quality of each match of the
%                        annotation (rows) with the computation (columns)
%   overlap_width ...... matrix containing the width of overlap for each
%                        annotation (rows) and computation (columns)
%   offset_start_idx ... matrix containing the offset in the start index
%                        for each annotation (rows) and computation (columns)
%   offset_end_idx ..... matrix containing the offset in the end index
%                        for each annotation (rows) and computation (columns)
%
% EXAMPLE
%   % tolerate 5 frames off in the overlap and use the overlap ratio
%   [Q,W,S,E] = evaluateSegmentation ( cutsm, cuts, 'OverlapRatio', 5);
%   % tolerate use the overlap-union ratio
%   [Q,W,S,E] = evaluateSegmentation ( cutsm, cuts, 'OverlapUnionRatio');
%

%'OverlapUnionRatio'
%'OverlapRatio'

numvarargs = length(varargin);
if numvarargs > 2
  error('evaluateSegmentation: TooManyInputs', ...
    'requires at most 2 optional inputs');
end % fi
optargs = {'OverlapUnionRatio', 60/5};
optargs(1:numvarargs) = varargin;

[similarity_measure, thresh] = optargs{:};


n_cutsm = numel(manual_cuts)/2;
n_cutsc = numel(computed_cuts)/2;

switch lower(similarity_measure)
  case 'numberofsegments'
    overlap_quality = n_cutsc/n_cutsm;
    if nargout > 1
      overlap_width = [];
      offset_start_idx = [];
      offset_end_idx = [];
    end
    return;
  otherwise
end

  %% pre-allocation
  overlap_width = zeros (n_cutsm, n_cutsc);
  overlap_quality = overlap_width;
  offset_start_idx = overlap_width;
  offset_end_idx = overlap_width;
  %% evaluation
  for kk=1:n_cutsm
    for kkk=1:n_cutsc
      
      o = computeIntervalOverlap(manual_cuts(kk,:),computed_cuts(kkk,:));

      if o ~= 0 % interval overlap is not zero
        
        % save current overlap width
        overlap_width(kk,kkk) = o;
        % save difference (offset) in start index for manual and computed cut
        offset_start_idx(kk,kkk) = manual_cuts(kk,1) - computed_cuts(kkk,1); 
        % difference (offset) in end index for manual and computed cut
        offset_end_idx(kk,kkk) = manual_cuts(kk,2) - computed_cuts(kkk,2); 


        
        switch lower(similarity_measure)
          case 'overlapunionratio'
            similarity = OverlapUnionRatio(manual_cuts(kk,:), computed_cuts(kkk,:), o);
          case 'overlapratio'
            similarity = OverlapRatio(manual_cuts(kk,:), computed_cuts(kkk,:), thresh, o);
          otherwise
        end %switch

        if similarity > 0.0
          overlap_quality(kk,kkk) = similarity;
        else
          overlap_quality(kk,kkk) = 0;
        end%fi
        
        
      else % interval overlap is zero
        overlap_width(kk,kkk) = 0;
        overlap_quality(kk,kkk) = 0;
        offset_start_idx(kk,kkk) = NaN;
        offset_end_idx(kk,kkk) = NaN;
      end % fi check interval overlap width
      
    end % for kkk=1:numel(cuts{k,1})/2
  end %for kk=1:numel(cutsm{k,1})/2
  % end of overlap computation
  
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function sim = OverlapUnionRatio ( cutm, cutc, overlap )
  % compute similarity of two intervals
  % it is defined as the ratio of overlap in width and width of union of
  % the underlying intervals. I.e., for for two intervals A, B over IN, 
  % this measure is computed as
  %   | A \cap B | / | A \cup B | 
  %
  % INPUT
  %   cutm ...... first interval
  %   cutc ...... second interval
  %   overlap ... width of the overlap of the two intervals
  % 
  % OUTPUT
  %   similarity ... value in [0, 1] telling how similar the two intervals
  %                  are. Higher values stand for higher similarity
  %
  % EXAMPLE
  %   sim = OverlapUnionRatio ( [10, 20], [5, 16] );
  %   sim = OverlapUnionRatio ( [10, 20], [5, 16], 7 );
  %
  
    if nargin == 2
      overlap = computeIntervalOverlap(cutm,cutc);
    end
    
    if overlap == 0
      sim = 0;
    else
      % width of current manual cut
      widthm = cutm(2) - cutm(1) + 1;
      % width of current computed cut
      widthc = cutc(2) - cutc(1) + 1;

      sim = overlap/(widthm+widthc-overlap);
      %sim = overlap/(widthm+widthc-overlap+1); % why +1 ? should not have to be +1
    end
  end % of function OverlapUnionRatio
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function sim = OverlapRatio ( cutm, cutc, thresh, overlap )
  % compute similarity of two intervals
  % it is defined as the ratio of overlap in width and width of union of
  % the underlying intervals. I.e., for for two intervals A, B over IN, 
  % this measure is computed as
  %   | A | / | B | if (|A| - |A \cap B|) < thresh 
  %
  % INPUT
  %   cutm ...... first interval
  %   cutc ...... second interval
  %   overlap ... width of the overlap of the two intervals
  % 
  % OUTPUT
  %   similarity ... value in [0, 1] telling how similar the two intervals
  %                  are. Higher values stand for higher similarity
  %
  % EXAMPLE
  %   sim = OverlapRatio ( [10, 20], [5, 16], Hz/5 );
  %   sim = OverlapRatio ( [10, 20], [5, 16], Hz/10, 7 );
  %
  
    if nargin == 2
      overlap = computeIntervalOverlap(cutm,cutc);
    end
    
    if overlap == 0
      sim = 0;
    else
      % width of current manual cut
      widthm = cutm(2) - cutm(1) + 1;
      % width of current computed cut
      widthc = cutc(2) - cutc(1) + 1;

      
      % This similarity measure makes only sense if the squares are nearly
      % overlapping, otherwise there's no meaning in the ratio of the
      % manual and computed segmentation
      % if the overlap and the manual annotation are less then a fifth 
      % of a seconds different (= outside the manual annotation) 
      % then compute similarity
      if (widthm-overlap) < thresh; 
        sim =  widthm/widthc;
        % in order to weigh in that the overlap is smaller than the width
        % of the current manual segment use the following 
        sim = (widthm/widthc+overlap/widthc)/2; 
      else
        sim = 0;
      end %fi  evaluate similarity only for suited segments

    end % fi check overlap

    
    
  end % of function OverlapRatio    
    


end % of function evaluateSegmentation
