function intervals_out = mergeIntervals (t, interval_list)
% intervals_out = mergeIntervals (tolerance, interval_list)
%
% Merge well formed intervals (i.e. for each interval [x1,x2] holds x1<x2)
% in an input list depending on an overlap tolerance level.
%
% For two intervals [x1,x2] and [y1,y2] the following cases apply
%   x2 < y1 => no overlap (=> x1,x2,y1,y2)
%   x1 > y2 => no overlap (=> y1,y2,x1,x2)
% Or, put the other way around, the intervals overlap if there is a value c 
% which is present in both intervals. I.e.
%   c > x1 && c < x2 (or x1 <= c <= x2)
%   &&
%   c > y1 && c < y2 (or y1 <= c <= y2)
% which forms a condition
%   x1 <= y2 && y1 <= x2
% 
%
% INPUT
%   tolerance ....... percentage of overlap up to which intervals are _not_ merged
%   interval_list ... cell array of lists of intervals to merge
%
% OUTPUT
%   intervals_out ... list of merged intervals containing 
%      start index, end index, width, origin, number of merged intervals
%
% EXAMPLE
%   % merge intervals that overlap up to 75% in width 
%   interv = mergeIntervals ( 0.75, list );
%
%
%
% Annotation:
%   The setup of the input should be
%      start end width origin
%   because the 3rd column will be overwritten with the width.
%
% origin indicates from which array the value came if it was combined from 
% multiple interval lists.
%
%

if isempty(interval_list)
  intervals_out = [];
  return;
end
% structure input
if iscell(interval_list)
  if sum(cellfun(@(x) isempty(x), interval_list))
    intervals_out = [];
    return;
  end
  nintervals = size(interval_list,2);
  for k=1:nintervals
    interval_list{k}(:,3) = interval_list{k}(:,2)-interval_list{k}(:,1)+1;% width
    interval_list{k}(:,4) = k; %interval list identifier
    interval_list{k}(:,5) = 1; %counter for counting merges
  end
  interval_list = vertcat(interval_list{:});
else
  temp(:,1) = interval_list(:,1); %start
  temp(:,2) = interval_list(:,2); %end
  temp(:,3) = interval_list(:,2)-interval_list(:,1)+1; %width
  temp(:,4) = 0; %unused in this case
  temp(:,5) = 1; %counter for merges
  interval_list = temp;
  clear temp;
end

interval_list = sortrows(interval_list);

% critical merge level (merge intervals by default when either overlaps the 
% other by more than a certain percentage (99%) unless the tolerance was 
% set higher)
cml = max(0.99,t);

intervals = interval_list(1,:);
for k=2:size(interval_list,1)
  next = interval_list(k,:);
  overlap = getOverlap_local(intervals(end,:), next);
  op0 = overlap/interval_list(k-1,3); % overlap in percentage with previous interval
  op1 = overlap/interval_list(k,3);   % overlap in percentage with current interval
  if (op0 > t && op1 > t) || (op0 >= cml || op1 >= cml)
    % merge if the two original intervals overlap by > tolerance or if one
    % overlaps the other by more than 99%
    intervals(end,2) = next(2);
    intervals(end,3) = intervals(end,2)-intervals(end,1)+1;
    intervals(end,4) = 0;
    intervals(end,5) = intervals(end,5)+1;
  else
    intervals = [intervals; next];
  end
  %debug_info(k-1,1) = overlap;
  %debug_info(k-1,2) = overlap/interval_list(k-1,3);
  %debug_info(k-1,3) = overlap/interval_list(k,3);
end
intervals_out = intervals(:,1:5);

if nargout==0
  cla
  plotSquares([intervals(:,1), intervals(:,2)], 'r');
  plotSquares([interval_list(:,1), interval_list(:,2)], 'b',1);
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function tf = getOverlap_local(A,B)
    tf = max ( -1, min(A(2),B(2)) - max(A(1),B(1)) ) + 1;
  end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function tf = doOverlap_local(A,B)
    tf = ( (A(1) <= B(2)) && (B(1) <= A(2)) );
  end %of function doOverlap
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end% of function